{------------------------------------------------------------------------------

Copyright (C) 2002  Rainer Queck
e-mail: Rainer.Queck@Qutronic.de
http://www.qutronic.de


This code is free; you can redistribute it and/or
modify it under the terms of the MPL MOZILLA PUBLIC LICENSE. 

This code is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
MPL MOZILLA PUBLIC LICENSE for more details at http://www.mozilla.org/MPL

//---------------------------------------------------------------------------}
// Unit Created at    : Date: 16.02.02  Time :13:17:26
// Unit last modified : Date: 21.12.2002  Time :10:13:30
//------------------------------------------------------------------------------
unit PrjNotify;

interface

uses SysUtils, Classes, MMToolsApi, Dialogs;

type
  TProjectNotifierObserver = class (TComponent)
  private
    FEnabled: Boolean;
    FOnAfterNew: TNotifyEvent;
    FOnAfterOpen: TNotifyEvent;
    FOnAfterSave: TNotifyEvent;
    FOnBeforeNew: TNotifyEvent;
    FOnBeforeOpen: TNotifyEvent;
    FOnBeforeSave: TNotifyEvent;
    FOnCommitEditors: TNotifyEvent;
    FOnDestroyed: TNotifyEvent;
  protected
    procedure AfterNewEvent;
    procedure AfterOpenEvent;
    procedure AfterSaveEvent;
    procedure BeforeNewEvent;
    procedure BeforeOpenEvent;
    procedure BeforeSaveEvent;
    procedure CommitEditorsEvent;
    procedure DestroyedEvent;
  published
    property Enabled: Boolean read FEnabled write FEnabled;
    property OnAfterNew: TNotifyEvent read FOnAfterNew write FOnAfterNew;
    property OnAfterOpen: TNotifyEvent read FOnAfterOpen write FOnAfterOpen;
    property OnAfterSave: TNotifyEvent read FOnAfterSave write FOnAfterSave;
    property OnBeforeNew: TNotifyEvent read FOnBeforeNew write FOnBeforeNew;
    property OnBeforeOpen: TNotifyEvent read FOnBeforeOpen write FOnBeforeOpen;
    property OnBeforeSave: TNotifyEvent read FOnBeforeSave write FOnBeforeSave;
    property OnCommitEditors: TNotifyEvent read FOnCommitEditors write 
            FOnCommitEditors;
    property OnDestroyed: TNotifyEvent read FOnDestroyed write FOnDestroyed;
  end;
  
  TProjectNotifier = class (TInterfacedObject, IUnknown, IMMNotifier, 
          IMMProjectNotifier)
  private
    FDoCancel: Boolean;
    FLog: TStrings;
    FObservers: TList;
  protected
    procedure AfterNewEvent;
    procedure AfterOpenEvent;
    procedure AfterSaveEvent;
    procedure BeforeNewEvent;
    procedure BeforeOpenEvent;
    procedure BeforeSaveEvent;
    procedure CommitEditorsEvent;
    procedure DestroyedEvent;
  public
    constructor Create;
    destructor Destroy; override;
    procedure AfterNew; safecall;
    procedure AfterOpen(const FileName: WideString); safecall;
    procedure AfterSave(const FileName: WideString); safecall;
    procedure BeforeNew(var Cancel: Boolean); safecall;
    procedure BeforeOpen(const FileName: WideString; var Cancel: Boolean); 
            safecall;
    procedure BeforeSave(const FileName: WideString; var Cancel: Boolean); 
            safecall;
    procedure CommitEditors; safecall;
    procedure Destroyed; safecall;
    procedure RegisterObserver(Observer: TProjectNotifierObserver);
    procedure UnregisterObserver(Observer: TProjectNotifierObserver);
    property DoCancel: Boolean read FDoCancel write FDoCancel;
    property Log: TStrings read FLog;
  end;
  

var
  ProjectNotifier: TProjectNotifier;

implementation

{
*************************** TProjectNotifierObserver ***************************
}
procedure TProjectNotifierObserver.AfterNewEvent;
begin
  if Assigned(FOnAfterNew) then FOnAfterNew(Self);
end;

procedure TProjectNotifierObserver.AfterOpenEvent;
begin
  if Assigned(FOnAfterOpen) then FOnAfterOpen(Self);
end;

procedure TProjectNotifierObserver.AfterSaveEvent;
begin
  if Assigned(FOnAfterSave) then FOnAfterSave(Self);
end;

procedure TProjectNotifierObserver.BeforeNewEvent;
begin
  if Assigned(FOnBeforeNew) then FOnBeforeNew(Self);
end;

procedure TProjectNotifierObserver.BeforeOpenEvent;
begin
  if Assigned(FOnBeforeOpen) then FOnBeforeOpen(Self);
end;

procedure TProjectNotifierObserver.BeforeSaveEvent;
begin
  if Assigned(FOnBeforeSave) then FOnBeforeSave(Self);
end;

procedure TProjectNotifierObserver.CommitEditorsEvent;
begin
  if Assigned(FOnCommitEditors) then FOnCommitEditors(Self);
end;

procedure TProjectNotifierObserver.DestroyedEvent;
begin
  if Assigned(FOnDestroyed) then FOnDestroyed(Self);
end;

{
******************************* TProjectNotifier *******************************
}
constructor TProjectNotifier.Create;
begin
  inherited Create;
  FLog := TStringList.Create;
  FObservers:=TList.Create;
end;

destructor TProjectNotifier.Destroy;
begin
  FLog.Free;
  FObservers.Free;
  if Self = ProjectNotifier then ProjectNotifier := nil;
  inherited Destroy;
end;

procedure TProjectNotifier.AfterNew;
begin
  AfterNewEvent;
end;

procedure TProjectNotifier.AfterNewEvent;
var
  I: Integer;
  Obs: TProjectNotifierObserver;
begin
  for I := 0 to FObservers.Count - 1 do
  begin
    Obs := FObservers[I];
    if Obs.Enabled then Obs.AfterNewEvent;
  end;
end;

procedure TProjectNotifier.AfterOpen(const FileName: WideString);
begin
  AfterOpenEvent;
end;

procedure TProjectNotifier.AfterOpenEvent;
var
  I: Integer;
  Obs: TProjectNotifierObserver;
begin
  for I := 0 to FObservers.Count - 1 do
  begin
    Obs := FObservers[I];
    if Obs.Enabled then Obs.AfterOpenEvent;
  end;
end;

procedure TProjectNotifier.AfterSave(const FileName: WideString);
begin
  AfterSaveEvent;
end;

procedure TProjectNotifier.AfterSaveEvent;
var
  I: Integer;
  Obs: TProjectNotifierObserver;
begin
  for I := 0 to FObservers.Count - 1 do
  begin
    Obs := FObservers[I];
    if Obs.Enabled then Obs.AfterSaveEvent;
  end;
end;

procedure TProjectNotifier.BeforeNew(var Cancel: Boolean);
begin
  BeforeNewEvent;
  Cancel := FDoCancel;
end;

procedure TProjectNotifier.BeforeNewEvent;
var
  I: Integer;
  Obs: TProjectNotifierObserver;
begin
  for I := 0 to FObservers.Count - 1 do
  begin
    Obs := FObservers[I];
    if Obs.Enabled then Obs.BeforeNewEvent;
  end;
end;

procedure TProjectNotifier.BeforeOpen(const FileName: WideString; var Cancel: 
        Boolean);
begin
  BeforeOpenEvent;
  Cancel := FDoCancel;
end;

procedure TProjectNotifier.BeforeOpenEvent;
var
  I: Integer;
  Obs: TProjectNotifierObserver;
begin
  for I := 0 to FObservers.Count - 1 do
  begin
    Obs := FObservers[I];
    if Obs.Enabled then Obs.BeforeOpenEvent;
  end;
end;

procedure TProjectNotifier.BeforeSave(const FileName: WideString; var Cancel: 
        Boolean);
begin
  BeforeSaveEvent;
  Cancel := FDoCancel;
end;

procedure TProjectNotifier.BeforeSaveEvent;
var
  I: Integer;
  Obs: TProjectNotifierObserver;
begin
  for I := 0 to FObservers.Count - 1 do
  begin
    Obs := FObservers[I];
    if Obs.Enabled then Obs.BeforeSaveEvent;
  end;
end;

procedure TProjectNotifier.CommitEditors;
begin
  CommitEditorsEvent;
end;

procedure TProjectNotifier.CommitEditorsEvent;
var
  I: Integer;
  Obs: TProjectNotifierObserver;
begin
  for I := 0 to FObservers.Count - 1 do
  begin
    Obs := FObservers[I];
    if Obs.Enabled then Obs.CommitEditorsEvent;
  end;
end;

procedure TProjectNotifier.Destroyed;
begin
  DestroyedEvent;
end;

procedure TProjectNotifier.DestroyedEvent;
var
  I: Integer;
  Obs: TProjectNotifierObserver;
begin
  for I := 0 to FObservers.Count - 1 do
  begin
    Obs := FObservers[I];
    if Obs.Enabled then Obs.DestroyedEvent;
  end;
end;

procedure TProjectNotifier.RegisterObserver(Observer: TProjectNotifierObserver);
begin
  if FObservers.IndexOf(Observer) = -1 then
    FObservers.Add(Observer);
end;

procedure TProjectNotifier.UnregisterObserver(Observer: 
        TProjectNotifierObserver);
begin
  FObservers.Remove(Observer);
end;

end.
